local Drawable = require 'graphics.drawable'
local List = require 'library.list'

local Container = Drawable:subclass('Container')

function Container:initialize()
    Drawable.initialize(self)

    self.masking = false
    self.children = List()

    local callbacks = {
        "update", "keypressed", "keyreleased", "textedited", "textinput",
        "mousemoved", "mousepressed", "mousereleased", "wheelmoved",
        "gamepadaxis", "gamepadpressed", "gamepadreleased", "joystickadded",
        "joystickaxis", "joystickhat", "joystickpressed", "joystickreleased",
        "joystickremoved", "touchmoved", "touchpressed", "touchreleased"
    }

    for _, v in ipairs(callbacks) do
        local func = function(...)
            local args = {...}
            self.children:forEach(function(i, child)
                if type(child[v]) == "function" then
                    child[v](unpack(args))
                end
            end)
        end

        self.class.__declaredMethods[v] = func
        self.class.__instanceDict[v] = func
    end
end

function Container:add(child)
    self.children:add(child)
end

function Container:addRange(children)
    self.children:addRange(children)
end

function Container:remove(index)
    self.children:remove(index)
end

function Container:removeAll(pred)
    self.children:remove(pred)
end

function Container:clearChildren()
    self.children:clear()
end

function Container:draw()
    if self.masking then
        love.graphics.stencil(function()
            love.graphics.rectangle("fill", self.x, self.y, self.w, self.h)
        end, "replace", 1)
        love.graphics.setStencilTest("greater", 0)
    end

    self.children:forEach(function(i, child)
        if (child.draw) then
            child:draw()
        end
    end)

    if self.masking then
        love.graphics.setStencilTest()
    end
end

return Container