local class = require 'library.middleclass'

local Drawable = class('Drawable')

function Drawable:initialize()
    self.x = x or 0
    self.y = y or 0
    self.w = 0
    self.h = 0
    self.a = 0
    self.sx = 1
    self.sy = 1
    self.kx = 0
    self.ky = 0
    self.parent = nil
    self.relativeSize = {
        w = false,
        h = false
    }
    self.relativePosition = {
        x = false,
        y = false
    }
    self.relativeShearing = {
        x = false,
        y = false
    }
    self.relativeRotation = false
end

function Drawable:_draw()
    love.graphics.push()
        if self.parent then
            if self.relativePosition.x and self.relativePosition.y then
                love.graphics.translate(self.parent.x, self.parent.y)
            elseif self.relativePosition.x then
                love.graphics.translate(self.parent.x, 0)
            elseif self.relativePosition.y then
                love.graphics.translate(0, self.parent.x)
            end

            if self.relativeRotation then
                love.graphics.rotate(self.parent.a)
            end

            if self.relativeShearing.x and self.relativeShearing.y then
                love.graphics.shear(self.parent.kx, self.parent.ky)
            elseif self.relativeShearing.x then
                love.graphics.shear(self.parent.kx, 0)
            elseif self.relativeShearing.y then
                love.graphics.shear(0, self.parent.kx)
            end
                
            if self.relativeSize.w then
                self.w = self.parent:getWidth() * self.w
            end

            if self.relativeSize.h then
                self.h = self.parent:getHeight() * self.h
            end
        end

        self:draw()
    love.graphics.pop()
end

--  Lets leave an empty draw function here for now
--  It's much more readable rather than to do a
--  condition check which will never likely happen
--  since we expect it's subclassed classes of it
--  is expected have their own

function Drawable:draw()

end

function Drawable:getX()
    if self.parent and self.relativePosition.x then
        return self.x + self.parent.x
    else
        return self.x
    end
end

function Drawable:getY()
    if self.parent and self.relativePosition.y then
        return self.y + self.parent.y
    else
        return self.y
    end
end

function Drawable:getPosition()
    return self:getX(), self:getY()
end

function Drawable:getWidth()
    return self.w * self.sx
end

function Drawable:getHeight()
    return self.h * self.sy
end

function Drawable:getDimensions()
    return self:getWidth(), self:getHeight()
end

function Drawable:rotate(deg)
    local rad = deg * (math.pi / 180)
    self.a = rad
end

return Drawable